==> keyboard_shortcuts-1.0.0.tar.gz: keyboard_shortcuts/src/Form/KeyboardShortcutsSettingsForm.php <==
<?php

namespace Drupal\keyboard_shortcuts\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

class KeyboardShortcutsSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'keyboard_shortcuts';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'keyboard_shortcuts.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('keyboard_shortcuts.settings');

    $roles = Role::loadMultiple();
    array_walk($roles, function(&$role) {
      $role = $role->label();
    });

    $form['#title'] = $this->t('Keyboard shortcut settings');
    $form['#tree'] = true;

    $form['shortcuts'] = [
      '#type' => 'fieldset',
      '#title' => 'Shortcuts',
      '#collapsed' => FALSE,
      '#description' => <<< QQ
        <p>For modifier keys you can use <em>shift</em>, <em>ctrl</em>, <em>alt</em>, or <em>meta</em>.</p>
        <p>You can substitute <em>option</em> for <em>alt</em> and <em>command</em> for <em>meta</em>.</p>
        <p>Other special keys are backspace, tab, enter, return, capslock, esc, escape, space, pageup, pagedown, end, home, left, up, right, down, ins, del, and plus.</p>
        <p>Any other key you should be able to reference by name like a, /, $, *, or =.</p>
        <h4>Single key</h4>
        <p>Just type in the character or special key names (see above).</p>
        <h4>Combination of keys</h4>
        <p>Combine several keys using a '+' sign (ctrl+s).</p>
        <h4>Sequence of keys</h4>
        <p>Define a sequence of keys using a space (d ctrl+s).</p>
        <h4>Tokens</h4>
        <p>You can use 'node' and 'user' tokens in the action argument fields.</p>
        <h4>Note</h4>
        <p>Some browsers block standard shortcuts like for example (ctrl+n or command+n). It is advised to use combinations that don't overwrite default browser behavior.</p>
QQ
    ];

    $shortcuts = $config->get('shortcuts') ?? [];

    // Filter out any empty keystrokes.
    $shortcuts = array_filter($shortcuts, function ( $value ) {
      return !empty($value['keystroke']);
    });

    // Add a new one to add
    $shortcuts[] = [
      'label' => 'Add new shortcut',
      'keystroke' => '',
      'enable_during_input' => 1,
      'prevent_default' => 1,
      'action' => 'redirect',
      'arguments' => [
        'type' => '',
        'selector' => '',
        'url' => '',
        'callback' => '',
      ],
    ];

    foreach ($shortcuts as $delta => $shortcut) {

      $auto_label = (empty($shortcut['keystroke']) ? 'Undefined shortcut' : $shortcut['keystroke']);
      $label = empty($shortcut['label']) ? $auto_label : $shortcut['label'];

      $form['shortcuts'][$delta] = array(
        '#type' => 'details',
        '#collapsed' => !empty($shortcut['keystroke']),
        '#collapsible' => TRUE,
        '#title' => ($label != $auto_label ? $label . ' (' . $auto_label . ')' : $label)
      );

      $form['shortcuts'][$delta]['label'] = array(
        '#type' => 'textfield',
        '#title' => 'Label',
        '#description' => 'Give the shortcut a label (used in the gui).',
        '#default_value' => $label,
      );

      $form['shortcuts'][$delta]['keystroke'] = array(
        '#type' => 'textfield',
        '#title' => 'Keystroke(s)',
        '#description' => 'Give the keystroke(s) to associate to this shortcut. Separate multiple keystrokes with a comma. Leave empty to remove this keystroke entry.',
        '#default_value' => $shortcut['keystroke'],
      );

      $form['shortcuts'][$delta]['roles'] = array(
        '#type' => 'checkboxes',
        '#title' => 'Roles',
        '#options' => $roles,
        '#description' => 'Select which roles are allowed to use this keystroke. Select none to disable this check.',
        '#default_value' => $shortcut['roles'],
      );

      $enable = $shortcut['enable_during_input'];
      $form['shortcuts'][$delta]['enable_during_input'] = array(
        '#type' => 'checkbox',
        '#title' => t('Enable during input'),
        '#description' => t('Enable the use of the shortcut inside of inputs (textarea, text, select, editable content)'),
        '#default_value' => $enable,
      );

      $enable = $shortcut['prevent_default'];
      $form['shortcuts'][$delta]['prevent_default'] = array(
        '#type' => 'checkbox',
        '#title' => t('Prevent default'),
        '#description' => t('Prevent default behaviour of the shortcut keystroke (eg. Ctrl+S, normally saves the webpage).'),
        '#default_value' => $enable,
      );

      $options = array(
        'trigger' => 'trigger',
        'modal' => 'modal',
        'redirect' => 'redirect',
        'callback' => 'callback',
      );
      $action = $shortcut['action'];
      $form['shortcuts'][$delta]['action'] = array(
        '#type' => 'select',
        '#title' => 'Action',
        '#description' => 'Give the action to be executed',
        '#default_value' => $action,
        '#options' => $options
      );

      $form['shortcuts'][$delta]['arguments'] = array(
        '#type' => 'fieldset',
        '#collapsed' => FALSE,
        '#collapsible' => FALSE,
        '#title' => 'Action settings'
      );

      $arguments = $shortcut['arguments'];

      // Trigger arguments
      $form['shortcuts'][$delta]['arguments']['type'] = array(
        '#type' => 'select',
        '#title' => 'Type',
        '#default_value' => $arguments['type'],
        '#options' => array(
          'click' => 'click',
          'mouseleave' => 'mouseleave',
          'mouseenter' => 'mouseenter',
        ),
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'trigger'],
          ],
        ],
      );
      $form['shortcuts'][$delta]['arguments']['selector'] = array(
        '#type' => 'textfield',
        '#title' => 'Selector',
        '#description' => 'Enter a jQuery style element selector. You can use tokens (see below).',
        '#default_value' => $arguments['selector'],
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'trigger'],
          ],
        ],
      );

      // 'redirect':
      $form['shortcuts'][$delta]['arguments']['url'] = array(
        '#type' => 'textfield',
        '#title' => 'URL',
        '#description' => 'Enter a url to redirect to. You can use tokens (see below).',
        '#default_value' => $arguments['url'],
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => [
              ['value' => 'redirect'],
              ['value' => 'modal']
            ],
          ],
        ],
      );

      // 'callback':
      $form['shortcuts'][$delta]['arguments']['callback'] = array(
        '#type' => 'textfield',
        '#title' => 'Callback',
        '#description' => 'The javascript callback to execute when the shortcut is pressed.',
        '#default_value' => $arguments['callback'],
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'callback'],
          ],
        ],
      );
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $config = $this->config('keyboard_shortcuts.settings');
    $shortcuts = $form_state->getValue('shortcuts');

    // Filter out any empty keystrokes.
    $shortcuts = array_filter($shortcuts, function ($shortcut ) {
      return !empty($shortcut['keystroke']);
    });

    // Clean up the arguments.
    array_walk($shortcuts, function(&$shortcut) {

      $arguments = [
        '' => [],
        'trigger' => ['url', 'callback'],
        'modal' => ['callback', 'type', 'selector'],
        'redirect' => ['callback', 'type', 'selector'],
        'callback' => ['url', 'type', 'selector'],
      ][$shortcut['action'] ?? ''];

      foreach($arguments as $argument) {
        unset($shortcut['arguments'][$argument]);
      }

    });

    $config->set('shortcuts',  $shortcuts);
    $config->save();

    parent::submitForm($form, $form_state);
  }

}

==> keyboard_shortcuts-1.0.1.tar.gz: keyboard_shortcuts/src/Form/KeyboardShortcutsSettingsForm.php <==
<?php

namespace Drupal\keyboard_shortcuts\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;

class KeyboardShortcutsSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'keyboard_shortcuts';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'keyboard_shortcuts.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('keyboard_shortcuts.settings');

    $roles = Role::loadMultiple();
    array_walk($roles, function(&$role) {
      $role = $role->label();
    });

    $form['#title'] = $this->t('Keyboard shortcut settings');
    $form['#tree'] = true;

    $form['shortcuts'] = [
      '#type' => 'fieldset',
      '#title' => 'Shortcuts',
      '#collapsed' => FALSE,
      '#description' => <<< QQ
        <p>For modifier keys you can use <em>shift</em>, <em>ctrl</em>, <em>alt</em>, or <em>meta</em>.</p>
        <p>You can substitute <em>option</em> for <em>alt</em> and <em>command</em> for <em>meta</em>.</p>
        <p>Other special keys are backspace, tab, enter, return, capslock, esc, escape, space, pageup, pagedown, end, home, left, up, right, down, ins, del, and plus.</p>
        <p>Any other key you should be able to reference by name like a, /, $, *, or =.</p>
        <h4>Single key</h4>
        <p>Just type in the character or special key names (see above).</p>
        <h4>Combination of keys</h4>
        <p>Combine several keys using a '+' sign (ctrl+s).</p>
        <h4>Sequence of keys</h4>
        <p>Define a sequence of keys using a space (d ctrl+s).</p>
        <h4>Tokens</h4>
        <p>You can use 'node' and 'user' tokens in the action argument fields.</p>
        <h4>Note</h4>
        <p>Some browsers block standard shortcuts like for example (ctrl+n or command+n). It is advised to use combinations that don't overwrite default browser behavior.</p>
QQ
    ];

    $shortcuts = $config->get('shortcuts') ?? [];

    // Filter out any empty keystrokes.
    $shortcuts = array_filter($shortcuts, function ( $value ) {
      return !empty($value['keystroke']);
    });

    // Add a new one to add
    $shortcuts[] = [
      'label' => 'Add new shortcut',
      'keystroke' => '',
      'roles' => [],
      'enable_during_input' => 1,
      'prevent_default' => 1,
      'action' => 'redirect',
      'arguments' => [
        'type' => '',
        'selector' => '',
        'url' => '',
        'callback' => '',
      ],
    ];

    foreach ($shortcuts as $delta => $shortcut) {

      $auto_label = (empty($shortcut['keystroke']) ? 'Undefined shortcut' : $shortcut['keystroke']);
      $label = empty($shortcut['label']) ? $auto_label : $shortcut['label'];

      $form['shortcuts'][$delta] = array(
        '#type' => 'details',
        '#collapsed' => !empty($shortcut['keystroke']),
        '#collapsible' => TRUE,
        '#title' => ($label != $auto_label ? $label . ' (' . $auto_label . ')' : $label)
      );

      $form['shortcuts'][$delta]['label'] = array(
        '#type' => 'textfield',
        '#title' => 'Label',
        '#description' => 'Give the shortcut a label (used in the gui).',
        '#default_value' => $label,
      );

      $form['shortcuts'][$delta]['keystroke'] = array(
        '#type' => 'textfield',
        '#title' => 'Keystroke(s)',
        '#description' => 'Give the keystroke(s) to associate to this shortcut. Separate multiple keystrokes with a comma. Leave empty to remove this keystroke entry.',
        '#default_value' => $shortcut['keystroke'] ?? '',
      );

      $form['shortcuts'][$delta]['roles'] = array(
        '#type' => 'checkboxes',
        '#title' => 'Roles',
        '#options' => $roles,
        '#description' => 'Select which roles are allowed to use this keystroke. Select none to disable this check.',
        '#default_value' => $shortcut['roles'] ?? [],
      );

      $enable = $shortcut['enable_during_input'] ?? 1;
      $form['shortcuts'][$delta]['enable_during_input'] = array(
        '#type' => 'checkbox',
        '#title' => t('Enable during input'),
        '#description' => t('Enable the use of the shortcut inside of inputs (textarea, text, select, editable content)'),
        '#default_value' => $enable,
      );

      $enable = $shortcut['prevent_default'] ?? 1;
      $form['shortcuts'][$delta]['prevent_default'] = array(
        '#type' => 'checkbox',
        '#title' => t('Prevent default'),
        '#description' => t('Prevent default behaviour of the shortcut keystroke (eg. Ctrl+S, normally saves the webpage).'),
        '#default_value' => $enable,
      );

      $options = array(
        'trigger' => 'trigger',
        'modal' => 'modal',
        'redirect' => 'redirect',
        'callback' => 'callback',
      );
      $action = $shortcut['action'] ?? 'redirect';
      $form['shortcuts'][$delta]['action'] = array(
        '#type' => 'select',
        '#title' => 'Action',
        '#description' => 'Give the action to be executed',
        '#default_value' => $action,
        '#options' => $options
      );

      $form['shortcuts'][$delta]['arguments'] = array(
        '#type' => 'fieldset',
        '#collapsed' => FALSE,
        '#collapsible' => FALSE,
        '#title' => 'Action settings'
      );

      $arguments = $shortcut['arguments'] ?? [];

      // Trigger arguments
      $form['shortcuts'][$delta]['arguments']['type'] = array(
        '#type' => 'select',
        '#title' => 'Type',
        '#default_value' => $arguments['type'] ?? '',
        '#options' => array(
          'click' => 'click',
          'mouseleave' => 'mouseleave',
          'mouseenter' => 'mouseenter',
        ),
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'trigger'],
          ],
        ],
      );
      $form['shortcuts'][$delta]['arguments']['selector'] = array(
        '#type' => 'textfield',
        '#title' => 'Selector',
        '#description' => 'Enter a jQuery style element selector. You can use tokens (see below).',
        '#default_value' => $arguments['selector'] ?? '',
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'trigger'],
          ],
        ],
      );

      // 'redirect':
      $form['shortcuts'][$delta]['arguments']['url'] = array(
        '#type' => 'textfield',
        '#title' => 'URL',
        '#description' => 'Enter a url to redirect to. You can use tokens (see below).',
        '#default_value' => $arguments['url'] ?? '',
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => [
              ['value' => 'redirect'],
              ['value' => 'modal']
            ],
          ],
        ],
      );

      // 'callback':
      $form['shortcuts'][$delta]['arguments']['callback'] = array(
        '#type' => 'textfield',
        '#title' => 'Callback',
        '#description' => 'The javascript callback to execute when the shortcut is pressed.',
        '#default_value' => $arguments['callback'] ?? '',
        '#states' => [
          'visible' => [
            ':input[name="keystrokes['.$delta.'][action]"]' => ['value' => 'callback'],
          ],
        ],
      );
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $config = $this->config('keyboard_shortcuts.settings');
    $shortcuts = $form_state->getValue('shortcuts');

    // Filter out any empty keystrokes.
    $shortcuts = array_filter($shortcuts, function ($shortcut ) {
      return !empty($shortcut['keystroke']);
    });

    // Clean up the arguments.
    array_walk($shortcuts, function(&$shortcut) {

      $arguments = [
        '' => [],
        'trigger' => ['url', 'callback'],
        'modal' => ['callback', 'type', 'selector'],
        'redirect' => ['callback', 'type', 'selector'],
        'callback' => ['url', 'type', 'selector'],
      ][$shortcut['action'] ?? ''];

      foreach($arguments as $argument) {
        unset($shortcut['arguments'][$argument]);
      }

    });

    $config->set('shortcuts',  $shortcuts);
    $config->save();

    parent::submitForm($form, $form_state);
  }

}
